(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: walk_gen.ml,v 1.13 2014/01/31 09:51:52 fclement Exp $ *)

open Format;;

(* An example of function to generate walker functions.
  The generator's concern is to generate pattern matchings and recursive
  calls to functions that walk the code. *)

module Gen : Walk_spec.Make =
 functor (Io : Walk_spec.Out) ->

struct

  let ppf = Io.buffer;;

  let comment s =
    (* Comments have already been printed by the data type printer. *)
    if Og_config.get_keep_comments () &&
       not (Og_config.get_keep_type_definitions ())
    then fprintf ppf "%s@ " (Og_types_pprint.comment s)
  ;;

  let directive s =
    fprintf ppf "%s;;@,@," s
  ;;

  let begin_file () =
    fprintf ppf
      "@[<v>(* Beginning of %s generated code *)@ @ "
      (Og_version.short)
  ;;

  let end_file () =
    fprintf ppf
      "(* End of %s generated code *)@]@."
      (Og_version.short)
  ;;

  let variable s = fprintf ppf "%s" s;;

(*  let begin_paren () = fprintf ppf "@[(wgbegin_paren";;
  let end_paren () = fprintf ppf "@,wgend_paren)@]";; *)
  let begin_paren () = fprintf ppf "@[(";;
  let end_paren () = fprintf ppf "@,)@]";;

  let begin_application () = fprintf ppf "@[<1>";;
    let separator_argument () = fprintf ppf "@ ";;
(*    let begin_argument () = fprintf ppf "(wgbegin_arg";;
    let end_argument () = fprintf ppf "wgend_arg)";; *)
    let begin_argument () = fprintf ppf "(";;
    let end_argument () = fprintf ppf ")";;
  let end_application () = fprintf ppf "@]";;

(*  let begin_tuple () = fprintf ppf "@[<1>(wgbegin_tuple";;*)
  let begin_tuple () = fprintf ppf "@[<1>(";;
    let separator_tuple () = fprintf ppf ",@ ";;
(*  let end_tuple () = fprintf ppf "wgend_tuple)@]";;*)
  let end_tuple () = fprintf ppf ")@]";;

  let from_input_module lab =
    Printf.sprintf "%s.%s" (Og_config.get_input_module_name ()) lab
  ;;

  let begin_record_pattern () = fprintf ppf "@[<hv>{@[<hv>";;
    let begin_record_field lab =
      fprintf ppf "@;<1 1>@[<2>%s =@ " (from_input_module lab);;
    let end_record_field () = fprintf ppf "@]";;
    let separator_record_field () = fprintf ppf ";";;
  let end_record_pattern () = fprintf ppf "@]@ }@] ->@,";;

  let begin_def rec_flag =
    fprintf ppf "@[<v 0>le@[t %s"
      (if rec_flag = Walker_ast.Rec then "rec " else "")
  ;;
    let begin_def_elem ((_ty_name, walker_name), ty_arguments) =
      variable walker_name;
      List.iter (function arg -> fprintf ppf " %s" arg) ty_arguments;
    ;;
    let end_def_elem () = fprintf ppf "@]";;
    let separator_def_elem () = fprintf ppf "@,@,an@[d ";;
  let end_def () = fprintf ppf "@,;;@,@]@ "
  ;;

(*  let begin_function () = fprintf ppf "@[<1>(wgbegin_fun@ ";;*)
  let begin_function () = fprintf ppf "@[<1>(fun@ ";;
    let begin_clause_pattern () = fprintf ppf "@[";;
    let end_clause_pattern () = fprintf ppf "@] ->";;
    let begin_clause_expr () = fprintf ppf "@ @[";;
    let end_clause_expr () =  fprintf ppf "@]";;
(*  let end_function () = fprintf ppf "wgend_fun)@]";;*)
  let end_function () = fprintf ppf ")@]";;

  let sum_tag lab = fprintf ppf "%s" (from_input_module lab);;

  let begin_sum_inj lab = fprintf ppf "%s " (from_input_module lab);;
  let end_sum_inj _s = ();;

  let begin_sum_clause () = fprintf ppf " =@ function@;<0 -1>@[";;
    let begin_sum_field () = fprintf ppf "@;<1 1>| @[";;
    let end_sum_field () = fprintf ppf "@]";;
  let end_sum_clause () = fprintf ppf "@]";;

  let begin_record_clause () =
    begin_sum_clause ();
    begin_sum_field ();;
  let end_record_clause () =
    end_sum_field ();
    end_sum_clause ();;

  let begin_alias_clause = begin_sum_clause;;
    let begin_aliased_clause = begin_sum_field;;
    let end_aliased_clause = end_sum_field;;
  let end_alias_clause = end_sum_clause;;

end
;;
